﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class EvidenceHandler : MonoBehaviour {
    
    //Lists to reference all the clues/statements/corroborations that exists in the current level.
    public List<Evidence> allClues;
    public List<Evidence> allStatements;
    public List<Evidence> allCorroborations;

    public CutsceneManager cutsceneManager;

    //List to reference all the possible case solutions.
    public List<CaseResult> allCaseResults;


    public Evidence[] evidenceList;

    //Lists to reference all the clues/statements/corroborations that the player has discovered in the current level.
    public List<Evidence> foundClues;
    public List<Evidence> foundStatements;
    public List<Evidence> foundCorroborations;

    public List<Evidence> potentialMatches;

    public CaseResult defaultResult;

    //A reference to our Null Evidence object.
    Evidence nEvidence;

    //Reference to timelineScript
    TimelineScript timelineScript;

    // Use this for initialization
    void Start ()
    {
        cutsceneManager = FindObjectOfType<CutsceneManager>() as CutsceneManager;

        defaultResult = this.GetComponent<CaseResult>();

        CaseResult[] caseResultObjs = FindObjectsOfType<CaseResult>();

        for (int i = 0; i < caseResultObjs.Length; i++)
        {
            allCaseResults.Add(caseResultObjs[i] as CaseResult);
        }

        //Object array to hold all of the evidence.
        evidenceList = FindObjectsOfType<Evidence>();

        allClues.Clear();
        allStatements.Clear();
        allCorroborations.Clear();

        //Loops through and pulls references to all the Evidence objects in the level, pushing them into their relevant lists.
        for (int i = 0; i < evidenceList.Length; i++)
        {
            //Create a new Evidence object that is equal to the current iteration in the Object array.
            Evidence g = evidenceList[i] as Evidence;

            //Tests the 'type' of the object and adds it the their relevant lists.
            if (g.type == Evidence.evidenceType.Clue)
                allClues.Add(g);
            else if (g.type == Evidence.evidenceType.Statement)
                allStatements.Add(g);
            else if (g.type == Evidence.evidenceType.Corroboration)
                allCorroborations.Add(g);
        }

        //Gets a reference to the null evidence object.
        nEvidence = this.gameObject.GetComponent<Evidence>();
    }
	
	// Update is called once per frame
	void Update ()
    {
        //When we call a corroboration (tbc);
		if(Input.GetButtonDown("COR"))
        {
            //Create a new piece of evidence that is equal to the return corroboration on two clues (In this case, pre-defined clue inputs).
            Evidence cor = testCorroboration(allClues[0], allClues[1]);

            //If that piece of evidence is NOT a NullEvidence type, add it to the list of found corroborrations.
            if (cor.type != Evidence.evidenceType.NullEvidence)
                foundCorroborations.Add(cor);
        }
	}


    public Evidence testCorroboration(Evidence e1, Evidence e2)
    {
        //Create a list of potential matches (corroborations that use the first evidence input as one of their pieces of evidence.
        potentialMatches.Clear();

        //For all corroborations...
        for (int i = allCorroborations.Count - 1; i >= 0; i--)
        {
            //Test if this corroboration's first item id is equal to the first evidence id input.
            if (allCorroborations[i].e1_id == e1.id)
            {
                //If yes, add the item to the list of potential corroborations.
                potentialMatches.Add(allCorroborations[i]);
            }

            //Test if this corroboration's second item id is equal to the first evidence id input.
            else if (allCorroborations[i].e2_id == e1.id)
            {
                //If yes, add the item to the list of potential corroborations.
                potentialMatches.Add(allCorroborations[i]);
            }
        }

        //For all potential corroborations...
        for (int i = 0; i < potentialMatches.Count; i++)
        {
            //If either of the current potential corroboration's evidence ID's match to the second evidence input...
            if (potentialMatches[i].e1_id == e2.id || potentialMatches[i].e2_id == e2.id)
            {
                //For all of the currently discovered corroborations...
                for(int j = 0; j < foundCorroborations.Count; j++)
                {
                    //If they match the current potential corroboration we are looking at...
                    if (foundCorroborations[j] == potentialMatches[i])
                    {
                        //Then the item already exists in the found corroborations list, therefore return Null Evidence; nothing new has been discovered.
                        return nEvidence;
                    }
                }

                //Otherwise, return the current item, as it is a match where the ID's of e1 and e2 match the ID's of the corroboration.
                return potentialMatches[i];
            }
        }

        //No matches have been discovered, so return Null Evidence.
        return nEvidence;
    }

    public Evidence GetEvidenceFromID(string id)
    {
        Evidence e = nEvidence;

        if (id.Contains("c"))
        {
            for (int i = 0; i < foundClues.Count; i++)
            {
                if (id == foundClues[i].id)
                    e = foundClues[i];
            }
        }

        if (id.Contains("s"))
        {
            for (int i = 0; i < foundStatements.Count; i++)
            {
                if (id == foundStatements[i].id)
                    e = foundStatements[i];
            }
        }

        if (id.Contains("e"))
        {
            for (int i = 0; i < foundCorroborations.Count; i++)
            {
                if (id == foundCorroborations[i].id)
                    e = foundCorroborations[i];
            }
        }

        return e;
    }

    /// <summary>
    /// This function adds an Evidence item to end of the case solution list.
    /// </summary>
    /// <param name="e">The Evidence item you wish to add.</param>
    public void AddEvidenceToCaseSolution(EvidenceUI e)
    {
        GameManager.instance.currentCaseSolution.Add(e);
    }

    /// <summary>
    /// This function adds an Evidence item to a specified position in the case solution list.
    /// </summary>
    /// <param name="e">The Evidence item you wish to add</param>
    /// <param name="pos">The index in the list you want the item to be placed.</param>
    public void AddEvidenceToCaseSolution(EvidenceUI e, int pos)
    {
        int count = GameManager.instance.currentCaseSolution.Count;
        if(pos > count)
        {
            //Error test.
            Debug.Log("ERROR: Cannot add Evidence Item " + e + " to the solution list because there are only " + count + " items currently in the list");
            return;
        }

        GameManager.instance.currentCaseSolution[pos] = e;
    }

    public bool CheckIfCaseSolutionIsFull()
    {
        int count = GameManager.instance.currentCaseSolution.Count;
        bool full = true;
        for(int i = 0; i < count; i++)
        {
            if(GameManager.instance.currentCaseSolution[i].thisEvidenceID == "")
            {
                full = false;
            }
        }
        return full;
    }

    public void chooseSolution()
    {
        timelineScript = FindObjectOfType<TimelineScript>();

        for (int i = 0; i < timelineScript.corroborations.Count; i++)
        {
            EvidenceUI curEv = timelineScript.corroborations[i].GetComponent<EvidenceUI>();
            AddEvidenceToCaseSolution(curEv);
        }

        CaseResult finalResult;

        List<CaseResult> potentialResults = new List<CaseResult>();

        List<float> minScores = new List<float>();
        List<float> scoreRatios = new List<float>();


        //Gets all the scores of the current solution;
        for (int i = 0; i < allCaseResults.Count; i++)
        {
            if (allCaseResults[i].canPass())
            {
                potentialResults.Add(allCaseResults[i]);
                minScores.Add(allCaseResults[i].requiredScoreToPass);
            }
        }

        int finalScoreIndex = 0;
        float finalScore = 0;

        for (int i = 0; i < potentialResults.Count; i++)
        {
            float s = ((potentialResults[i].calculateScore() - potentialResults[i].requiredScoreToPass) /
                        (100 - potentialResults[i].requiredScoreToPass)) *
                        100.0f;

            if (s > finalScore)
            {
                finalScore = s;
                finalScoreIndex = i;
            }
        }

        finalResult = potentialResults[finalScoreIndex];

        cutsceneManager.startCutscene(finalResult.endCutsceneID);
    }
}
